/*
This sketch reads a Sharp IR distance sensor on pin A0 and a PIR motion sensor on pin D2.
It sends the digital sensor value out to Max via the serial port.
It converts the analog sensor value to a linear distance, then breaks the 10-bit value into low- and high-bytes and sends them to Max via the serial port.

Adapted from Aruino Cookbook 2E recipes 6.3 & 6.5 along with my own sm_max_arduino_serial examples 1 & 2.

Steven M. Miller
*/

const int DInputPin = 2;            // Set the digial sensor input pin to 2 (for the PIR motion sensor)
const int AInputPin = 0;            // Set the analog sensor input pin to 0 (for IR disance sensor)
const long referenceMv = 5000;      // Long int to prevent overflow when multiplied (for scaling the IR sensor voltage)


int Val = 0;                        // Variable for current value read from sensor pin (0-1 digital; 0-1023 analog)
int mV = 0;                         // Variable for analog sensor value converted to millivolts (0-5000)
int cm = 0;                         // Variable for mV converted to distance (in cm) (15-150)
int APrevVal = 0;                   // Variable for previous distance value (in cm)
int NewVal = 0;                     // Variable for averaged distance value ((current + previous) / 2) (in cm)
int InByte = 0;                     // Variable for incoming serial data

void setup()
{
  Serial.begin(9600);               // Set communication speed
  pinMode(DInputPin, INPUT);        // Declare pin D2 as input for PIR sensor
}

void loop()
{
  if(Serial.available()>0)          // Is there data waiting in the serial port? (this data is from Max for hand-shaking)
  {                                 // If so, proceed with code block, otherwise jump directly to end of loop (and repeat)
  InByte = Serial.read();           // Read data from serial buffer and store in variable (to get rid of it)
  
  Val = digitalRead(DInputPin);     // Read digital sensor value and store in Val
  Serial.write(Val);                // Send the value out the serial port
  
  Val = analogRead(AInputPin);      // Read analog sensor value and store in Val
  mV = (Val * referenceMv) / 1023;  // Scale the sensor value (0-1023) to millivolts range (0-5000)
  cm = getDistance(mV);             // Convert mV (0-5000) to cm (15-150) via 'getDistance' function defined below
  NewVal = (cm + APrevVal) / 2;     // Average current and previous cm values for smoothing
  APrevVal = cm;                    // Store current cm value as PrevVal for next time
  SendBinary(NewVal);               // Break NewVal into low & high bytes and send out serial port via 'SendBinary' function defined below

  Serial.println();                 // Send CR & LF (ASCII 13 & 10) to mark end of message
  }
}

// function to send the given integer value to the serial port as two bytes
void SendBinary(int value)
{
  // send the two bytes that comprise an integer
  Serial.write(lowByte(value));    // send the low byte
  Serial.write(highByte(value));   // send the high byte
}

// the following is used to interpolate the distance from a table
// table entries are distances in steps of 250 millivolts
const int TABLE_ENTRIES = 12;                                                   // Table size
const int firstElement = 250;                                                   // First entry is analogous to 250 mV
const int INTERVAL  = 250;                                                      // Interval size (in millivolts) between each table entry
static int distance[TABLE_ENTRIES] = {150,140,130,100,60,50,40,35,30,25,20,15}; // Table of distance values (in cm)

// Function which converts mV to cm via lookup table
int getDistance(int mV)
{
   if( mV >  INTERVAL * TABLE_ENTRIES-1 )                                       // Is mV value beyond the second-to-last (table size - 1) entry of the table?
      return distance[TABLE_ENTRIES-1];                                         // If so, return last table value
   else                                                                         // If not, proceed with table lookup and conversion
   {
      int index = mV / INTERVAL;                                                // Index into table = mV / interval size (250)
      float frac = (mV % 250) / (float)INTERVAL;                                // Find interpolation value, to calculate values between table entries
      return distance[index] - ((distance[index] - distance[index+1]) * frac);  // Use interpolation value to calculate cm value between two table entries; return cm value
   }
}
